//***********************************************************************************
//* Class for Rossler model ODE*/
//***********************************************************************************

public class RosslerFunction extends OdesFunction {


/** the default value of the initial value of x */
 
      
      RosslerFunction(int n) {
            super(n);
            x0=new double[n];
            x0[0]=0.;
            x0[1]=1.;
            x0[2]=2.;
            x0[3]=1.5;
            dt=0.1;
            trans=20.;
            ghostTime=20.;
            poincareSection=1.;
            title="Rossler Model";
            nParameters=3;
            aDefault=new double[nParameters];
            aDefault[0]=10.;
            aDefault[1]=0.15;
            aDefault[2]=0.2;
            a=new double[nParameters];          
            
      }      
      
//***********************************************************************************
/** Sets the eqiation paramters 
* @param paramters the array of input paramters
*/
//***********************************************************************************      
      public void setParameters(double[] parameters) {
            for(int i=0; i<nParameters;i++) a[i]=parameters[i];
      }

//*********************************************************************
/**
* Returns RHS of differential equations for Rossler model
* @param x[] vector of current value of dependent variables
* @param n number of dependent variables in array x[]
* @param t current value of independent variable
* @return n compoent vector giving derivatives of dependent variables
*/
//*********************************************************************       
      public double[] derivs(double[] x, double t){
           double rhs[] = new double[nVariables];
           rhs[0]=1.;
           rhs[1]=x[2]-x[3];
           rhs[2]=-x[1]+a[1]*x[2];
           rhs[3]=a[2]+x[3]*(x[1]-a[0])  ;
           return rhs;
      }

     
//***********************************************************************************
/** Returnd the number of paraemters defining the map
* @return number of parameters
*/
//***********************************************************************************
     public int getNParameters() {
            return nParameters;
     } 
                  
}
